--- app_authenticate.c.orig	2005-06-07 02:39:31.000000000 +0400
+++ app_authenticate.c	2005-07-21 11:14:20.402689231 +0400
@@ -46,6 +46,10 @@
 "of the following letters:\n"
 "     a - Set account code to the password that is entered\n"
 "     d - Interpret path as database key, not literal file\n"
+"     m - Interpret path as a file which contains a list of\n"
+"         account codes and password hashes delimited with ':'\n"
+"         one per line. When password matched, corresponding\n"
+"         account code will be set\n"
 "     j - Support jumping to n+101\n"
 "     r - Remove database key upon successful entry (valid with 'd' only)\n"
 "\n"
@@ -110,25 +114,60 @@
 					break;
 				}
 			} else {
-				/* Compare against a file */
-				FILE *f;
-				f = fopen(password, "r");
-				if (f) {
-					char buf[256] = "";
-					while(!feof(f)) {
-						fgets(buf, sizeof(buf), f);
-						if (!feof(f) && !ast_strlen_zero(buf)) {
-							buf[strlen(buf) - 1] = '\0';
-							if (!ast_strlen_zero(buf) && !strcmp(passwd, buf))
-								break;
-						}
-					}
-					fclose(f);
-					if (!ast_strlen_zero(buf) && !strcmp(passwd, buf))
-						break;
-				} else 
+				if(strchr(opts, 'm')) {
+				    /* Compare against a file */
+				    FILE *f;
+				    f = fopen(password, "r");
+				    if (f) {
+					    char buf[256] = "";
+					    char md5passwd[17] = "";
+					    char *md5secret;
+					    while(!feof(f)) {
+						    fgets(buf, sizeof(buf), f);
+						    if (!feof(f) && !ast_strlen_zero(buf)) {
+							    buf[strlen(buf) - 1] = '\0';
+							    if (!ast_strlen_zero(buf)) {
+								md5secret = strchr(buf, ':');
+								if(!md5secret)
+								    continue;
+								*md5secret = '\0';
+								md5secret++;
+								ast_md5_hash(md5passwd, passwd);
+							        if(!ast_strlen_zero(buf) && !strcmp(md5passwd, md5secret)) {
+								    ast_cdr_setaccount(chan, buf);
+								    break;
+								}
+							    }
+						    }
+					    }
+					    fclose(f);
+					    if (!ast_strlen_zero(buf) && !strcmp(md5passwd, md5secret))
+						    break;
+				    } else 
+					ast_log(LOG_WARNING, "Unable to open file '%s' for authentication: %s\n", password, strerror(errno));
+
+				} else {
+				    /* Compare against a file */
+				    FILE *f;
+				    f = fopen(password, "r");
+				    if (f) {
+					    char buf[256] = "";
+					    while(!feof(f)) {
+						    fgets(buf, sizeof(buf), f);
+						    if (!feof(f) && !ast_strlen_zero(buf)) {
+							    buf[strlen(buf) - 1] = '\0';
+							    if (!ast_strlen_zero(buf) && !strcmp(passwd, buf))
+								    break;
+						    }
+					    }
+					    fclose(f);
+					    if (!ast_strlen_zero(buf) && !strcmp(passwd, buf))
+						    break;
+				    } else 
 					ast_log(LOG_WARNING, "Unable to open file '%s' for authentication: %s\n", password, strerror(errno));
+			      }
 			}
+			
 		} else {
 			/* Compare against a fixed password */
 			if (!strcmp(passwd, password)) 
@@ -147,7 +186,7 @@
 			chan->priority+=100;
 			res = 0;
 		} else {
-			if (!ast_streamfile(chan, "vm-goodbye", chan->language))
+			if (!ast_streamfile(chan, "auth-goodbye", chan->language))
 				res = ast_waitstream(chan, "");
 			res = -1;
 		}
